ENCODING = "utf-8"


def from_snake_to_pascal(snake_str: str) -> str:
    """Convert snake_case to PascalCase."""
    return "".join(x.capitalize() for x in snake_str.lower().split("_"))


def from_snake_to_camel(snake_str: str) -> str:
    """Convert snake_case to camelCase."""
    if "_" not in snake_str:
        return snake_str
    camel_string = from_snake_to_pascal(snake_str)
    return snake_str[0].lower() + camel_string[1:]



from typing import Literal, Optional

from pydantic import Field

from telegram_wallet_pay.enums import Currency

from ._default import DefaultModel
from .money_amount import MoneyAmount


class CreateOrderRequest(DefaultModel):
    amount: MoneyAmount
    auto_conversion_currency: Optional[
        Literal[
            Currency.TON,
            Currency.NOT,
            Currency.BTC,
            Currency.USDT,
        ]
    ] = None
    description: str = Field(min_length=5, max_length=100)
    return_url: Optional[str] = Field(None, max_length=255)
    fail_return_url: Optional[str] = Field(None, max_length=255)
    custom_data: Optional[str] = Field(None, max_length=255)
    external_id: str = Field(max_length=255)
    timeout_seconds: int = Field(ge=30, le=864000)
    customer_telegram_user_id: int



import asyncio
import os

from telegram_wallet_pay import TelegramWalletPay

# store TELEGRAM_WALLET_PAY_TOKEN to your .env
# wallet token can be issued via https://pay.wallet.tg/
TOKEN = os.getenv("TELEGRAM_WALLET_PAY_TOKEN")


async def main() -> None:
    """Get order preview."""
    # create wallet client instance
    wallet = TelegramWalletPay(TOKEN)

    # get order preview
    response = await wallet.get_order_preview("<your-order-id>")

    # let's print received response
    print("Response:", response)
    print("Order Preview:", response.data)

    # don't forget close API-client instance on your app shutdown
    await wallet.close()


if __name__ == "__main__":
    asyncio.run(main())



__all__ = [
    "ButtonName",
    "Currency",
    "OrderStatus",
    "RequestStatus",
    "WebhookMessageType",
]

from .button_name import ButtonName
from .currency import Currency
from .order_status import OrderStatus
from .request_status import RequestStatus
from .webhook_message_type import WebhookMessageType



from datetime import datetime
from typing import Any

import pytest
from pydantic import BaseModel, RootModel
from telegram_wallet_pay.schemas import (
    MoneyAmount,
    PaymentOption,
    WebhookMessage,
    WebhookMessages,
    WebhookPayload,
)

money_amount = MoneyAmount(currency_code="EUR", amount="42")
payment_option = PaymentOption(
    amount=money_amount,
    amount_fee=money_amount,
    amount_net=money_amount,
    exchange_rate="",
)
webhook_payload = WebhookPayload(
    id=1,
    number="",
    external_id="",
    order_amount=money_amount,
    selected_payment_option=payment_option,
    order_completed_datetime=datetime.now(),
)
webhook_message = WebhookMessage(
    event_datetime=datetime.now(),
    event_id=1,
    type="ORDER_PAID",
    payload=webhook_payload,
)


@pytest.mark.parametrize(
    ("schema", "data"),
    [
        (WebhookMessages, [webhook_message.model_dump(by_alias=True)]),
    ],
)
def test_iteration(schema: RootModel, data: list[dict[str, Any]]) -> None:
    """Test object is iterable."""
    iterable_object = schema.model_validate(data)
    for child in iterable_object:
        assert isinstance(child, (RootModel, BaseModel))



import json
from datetime import datetime
from uuid import uuid4

import pytest
from telegram_wallet_pay.tools import compute_signature

from tests.samples import WEBHOOK_MESSAGES


@pytest.fixture
def token() -> str:
    """Prepare token."""
    return str(uuid4())


@pytest.fixture
def path() -> str:
    """Prepare path."""
    return "/wallet"


@pytest.fixture
def body() -> str:
    """Prepare body."""
    return json.dumps(WEBHOOK_MESSAGES)


@pytest.fixture
def timestamp() -> str:
    """Prepare timestamp."""
    return str(datetime.now().timestamp())


@pytest.fixture
def signature(timestamp: str, body: str, token: str, path: str) -> str:
    """Prepare signature."""
    return compute_signature(
        store_api_key=token,
        http_method="POST",
        uri_path=path,
        timestamp=timestamp,
        body=body,
    )



from collections.abc import Iterator
from datetime import datetime
from typing import Literal

from pydantic import Field

from telegram_wallet_pay.enums import WebhookMessageType

from ._default import DefaultModel, DefaultRootModel
from .webhook_payload import WebhookPayload


class WebhookMessage(DefaultModel):
    event_datetime: datetime = Field(alias="eventDateTime")
    event_id: int
    type: Literal[
        WebhookMessageType.ORDER_PAID,
        WebhookMessageType.ORDER_FAILED,
    ]
    payload: WebhookPayload


class WebhookMessages(DefaultRootModel):
    root: list[WebhookMessage]

    def __iter__(self) -> Iterator[WebhookMessage]:  # type: ignore[override]
        """Iterate over root model."""
        return iter(self.root)



from pydantic import BaseModel, ConfigDict, RootModel

from telegram_wallet_pay.tools import from_snake_to_camel

_model_config = ConfigDict(
    use_enum_values=True,
    alias_generator=from_snake_to_camel,
    populate_by_name=True,
    validate_assignment=True,
    arbitrary_types_allowed=True,
)


class DefaultModel(BaseModel):
    model_config = _model_config


class DefaultRootModel(RootModel):
    model_config = _model_config  # noqa: pyCharm bug



from ipaddress import IPv4Address, IPv4Network
from typing import Union

import pytest
from telegram_wallet_pay.tools.ip_filter import DEFAULT_WALLET_WEBHOOK_IPS, IPFilter


def test_default_init() -> None:
    ip_filter = IPFilter()
    for ip in DEFAULT_WALLET_WEBHOOK_IPS:
        assert ip_filter.check(ip)


@pytest.mark.parametrize(
    ("ip", "result"),
    [
        ("127.0.0.1", True),
        ("127.0.0.2", False),
        (IPv4Address("127.0.0.1"), True),
        (IPv4Address("127.0.0.2"), False),
        (IPv4Address("192.168.0.32"), True),
        ("192.168.0.33", False),
        ("10.111.0.5", True),
        ("10.111.0.100", True),
        ("10.111.1.100", False),
    ],
)
def test_check_ip(ip: Union[str, IPv4Address], result: bool) -> None:
    ip_filter = IPFilter(
        ips=[
            "127.0.0.1",
            IPv4Address("192.168.0.32"),
            IPv4Network("10.111.0.0/24"),
        ],
    )
    assert (ip in ip_filter) is result


@pytest.mark.parametrize(
    ("ip", "ip_range"),
    [
        ("127.0.0.1", {IPv4Address("127.0.0.1")}),
        ("91.108.4.0/22", set(IPv4Network("91.108.4.0/22").hosts())),
        (IPv4Address("91.108.4.5"), {IPv4Address("91.108.4.5")}),
        (IPv4Network("91.108.4.0/22"), set(IPv4Network("91.108.4.0/22").hosts())),
    ],
)
def test_allow_ip(ip: str, ip_range: set[IPv4Address]) -> None:
    ip_filter = IPFilter([])
    ip_filter.allow_ip(ip)
    assert ip_filter._allowed_ips == ip_range


def test_allow_wrong_ip() -> None:
    ip_filter = IPFilter([])
    with pytest.raises(TypeError):
        # noinspection PyTypeChecker
        ip_filter.allow_ip(42)



import asyncio
import ssl
import warnings
from collections.abc import AsyncIterator, Mapping
from contextlib import asynccontextmanager
from decimal import Decimal
from http import HTTPStatus
from typing import (
    Any,
    Literal,
    Optional,
    TypeVar,
    Union,
)

import certifi
from aiohttp import ClientResponse, ClientSession, TCPConnector
from pydantic import BaseModel

from telegram_wallet_pay.enums import Currency
from telegram_wallet_pay.errors import (
    InvalidAPIKeyError,
    InvalidRequestError,
    NotFountError,
    RequestLimitReachedError,
    TelegramWalletPayError,
    UnexpectedError,
)
from telegram_wallet_pay.schemas import (
    CreateOrderRequest,
    CreateOrderResponse,
    GetOrderPreviewResponse,
    GetOrderReconciliationListResponse,
    MoneyAmount,
    OrderAmountResponse,
)

T = TypeVar("T", bound=BaseModel)

AUTH_HEADER = "Wpay-Store-Api-Key"
DEFAULT_API_HOST = "https://pay.wallet.tg"

EXCEPTIONS_MAPPING: dict[Union[HTTPStatus, int], type[TelegramWalletPayError]] = {
    HTTPStatus.BAD_REQUEST: InvalidRequestError,
    HTTPStatus.UNAUTHORIZED: InvalidAPIKeyError,
    HTTPStatus.NOT_FOUND: NotFountError,
    HTTPStatus.TOO_MANY_REQUESTS: RequestLimitReachedError,
    HTTPStatus.INTERNAL_SERVER_ERROR: UnexpectedError,
}


class TelegramWalletPay:
    """Telegram Wallet API client."""

    def __init__(self, token: str, api_host: str = DEFAULT_API_HOST) -> None:
        if not token or not isinstance(token, str):
            msg = f"String token should be provided. You passed: {token}"
            raise RuntimeError(msg)

        self._base_url = api_host
        self._session: Optional[ClientSession] = None
        self._headers = {AUTH_HEADER: token}

    async def create_order(  # noqa: PLR0913
        self,
        *,
        amount: Union[str, Decimal, float],
        currency_code: Literal[
            Currency.TON,
            Currency.NOT,
            Currency.BTC,
            Currency.USDT,
            Currency.EUR,
            Currency.USD,
            Currency.RUB,
        ],
        description: str,
        external_id: str,
        timeout_seconds: int,
        customer_telegram_user_id: int,
        auto_conversion_currency: Optional[
            Literal[
                Currency.TON,
                Currency.NOT,
                Currency.BTC,
                Currency.USDT,
            ]
        ] = None,
        return_url: Optional[str] = None,
        fail_return_url: Optional[str] = None,
        custom_data: Optional[str] = None,
    ) -> CreateOrderResponse:
        """Create an order.

        Docs:
        https://docs.wallet.tg/pay/#tag/Order/operation/create
        """
        create_order_request = CreateOrderRequest(
            amount=MoneyAmount(
                amount=str(amount),
                currency_code=currency_code,
            ),
            auto_conversion_currency=auto_conversion_currency,
            description=description,
            return_url=return_url,
            fail_return_url=fail_return_url,
            custom_data=custom_data,
            external_id=external_id,
            timeout_seconds=timeout_seconds,
            customer_telegram_user_id=customer_telegram_user_id,
        )

        async with self._make_request(
            method="POST",
            url="/wpay/store-api/v1/order",
            json=create_order_request.model_dump(by_alias=True),
        ) as response:
            return await self._prepare_result(response, CreateOrderResponse)

    async def get_preview(self, order_id: str) -> GetOrderPreviewResponse:
        """Retrieve the order information.

        Deprecated! Use method `.get_order_preview()` instead.
        """
        warnings.warn(
            "Method `.get_preview()` is deprecated and will be removed in v1.0.0\n"
            "Use method `.get_order_preview()` instead.",
            category=DeprecationWarning,
            stacklevel=2,
        )
        return await self.get_order_preview(order_id)

    async def get_order_preview(self, order_id: str) -> GetOrderPreviewResponse:
        """Retrieve the order information.

        Docs:
        https://docs.wallet.tg/pay/#tag/Order/operation/getPreview
        """
        async with self._make_request(
            method="GET",
            url="/wpay/store-api/v1/order/preview",
            params={"id": order_id},
        ) as response:
            return await self._prepare_result(response, GetOrderPreviewResponse)

    async def get_order_list(
        self,
        *,
        offset: int,
        count: int,
    ) -> GetOrderReconciliationListResponse:
        """Get list of store orders.

        Items sorted by creation time in ascending order.

        Docs:
        https://docs.wallet.tg/pay/#tag/Order-Reconciliation/operation/getOrderList
        """
        query_params: dict[str, Any] = {
            "offset": offset,
            "count": count,
        }

        async with self._make_request(
            method="GET",
            url="/wpay/store-api/v1/reconciliation/order-list",
            params=query_params,
        ) as response:
            return await self._prepare_result(
                response,
                GetOrderReconciliationListResponse,
            )

    async def get_order_amount(self) -> OrderAmountResponse:
        """Get total count of all created orders in the Store.

        Including all - paid and unpaid.

        Docs:
        https://docs.wallet.tg/pay/#tag/Order-Reconciliation/operation/getOrderAmount
        """
        async with self._make_request(
            method="GET",
            url="/wpay/store-api/v1/reconciliation/order-amount",
        ) as response:
            return await self._prepare_result(response, OrderAmountResponse)

    async def close(self) -> None:
        """Graceful session close."""
        if not self._session:
            return

        await self._session.close()

        # Wait 250 ms for the underlying SSL connections to close
        # https://docs.aiohttp.org/en/stable/client_advanced.html#graceful-shutdown
        await asyncio.sleep(0.25)

    async def _get_session(self) -> ClientSession:
        """Get aiohttp session with cache."""
        if self._session is None or self._session.closed:
            ssl_context = ssl.create_default_context(cafile=certifi.where())
            connector = TCPConnector(ssl=ssl_context)
            self._session = ClientSession(
                base_url=self._base_url,
                connector=connector,
                headers=self._headers,
            )

        return self._session

    @asynccontextmanager
    async def _make_request(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, str]] = None,
        json: Optional[Mapping[str, str]] = None,
    ) -> AsyncIterator[ClientResponse]:
        """Make request with cached session."""
        session = await self._get_session()
        async with session.request(
            method=method,
            url=url,
            params=params,
            json=json,
        ) as response:
            yield response

    @staticmethod
    async def _prepare_result(response: ClientResponse, schema: type[T]) -> T:
        """Prepare response result or raise an exception."""
        status = response.status
        body = await response.text()

        if status == HTTPStatus.OK:
            return schema.model_validate_json(body)

        exc_type = EXCEPTIONS_MAPPING.get(status, TelegramWalletPayError)
        raise exc_type(body)



import json
from collections.abc import Awaitable
from functools import wraps
from secrets import compare_digest
from typing import Callable, Optional, TypeVar

from aiohttp import web

from .signature import compute_signature

Request = TypeVar("Request", bound=web.Request)
Response = TypeVar("Response", bound=web.StreamResponse)
Handler = Callable[..., Awaitable[Response]]


def get_signature(request: web.Request) -> str:
    """Get signature from header."""
    try:
        signature = request.headers["WalletPay-Signature"]
    except KeyError:
        raise web.HTTPBadRequest(
            text=json.dumps({"detail": "Signature header is missing"}),
            content_type="application/json",
        ) from None

    if not signature:
        raise web.HTTPBadRequest(
            text=json.dumps({"detail": "Signature header is empty"}),
            content_type="application/json",
        )

    return signature


def get_timestamp(request: web.Request) -> str:
    """Get timestamp from header."""
    try:
        timestamp = request.headers["WalletPay-Timestamp"]
    except KeyError:
        raise web.HTTPBadRequest(
            text=json.dumps({"detail": "Timestamp header is missing"}),
            content_type="application/json",
        ) from None

    if not timestamp:
        raise web.HTTPBadRequest(
            text=json.dumps({"detail": "Timestamp header is empty"}),
            content_type="application/json",
        )

    return timestamp


def check_signature(
    store_api_key: str,
    custom_exception: Optional[web.HTTPClientError] = None,
) -> Callable:
    """Decorate aiohttp handler to check signature first."""
    exception = custom_exception or web.HTTPBadRequest(
        text=json.dumps({"detail": "Signature is not valid"}),
        content_type="application/json",
    )

    def decorator(handler: Handler) -> Handler:
        @wraps(handler)
        async def wrapper(
            request: web.Request,
            *args,
            **kwargs,
        ) -> Response:
            """Check signature."""
            signature = get_signature(request)
            timestamp = get_timestamp(request)
            raw_body = await request.text()

            valid_signature = compute_signature(
                store_api_key=store_api_key,
                http_method=request.method,
                uri_path=request.url.path,
                timestamp=timestamp,
                body=raw_body,
            )

            if not compare_digest(signature, valid_signature):
                raise exception

            return await handler(request, *args, **kwargs)

        return wrapper

    return decorator



__all__ = [
    "DefaultModel",
    "DefaultRootModel",
    "CreateOrderRequest",
    "CreateOrderResponse",
    "GetOrderPreviewResponse",
    "GetOrderReconciliationListResponse",
    "MoneyAmount",
    "OrderAmount",
    "OrderAmountResponse",
    "OrderPreview",
    "OrderReconciliationItem",
    "OrderReconciliationList",
    "PaymentOption",
    "WebhookMessage",
    "WebhookMessages",
    "WebhookPayload",
]

from ._default import DefaultModel, DefaultRootModel
from .create_order_request import CreateOrderRequest
from .create_order_response import CreateOrderResponse
from .get_order_preview_response import GetOrderPreviewResponse
from .get_order_reconciliation_list_response import GetOrderReconciliationListResponse
from .money_amount import MoneyAmount
from .order_amount import OrderAmount
from .order_amount_response import OrderAmountResponse
from .order_preview import OrderPreview
from .order_reconciliation_item import OrderReconciliationItem
from .order_reconciliation_list import OrderReconciliationList
from .payment_option import PaymentOption
from .webhook_message import WebhookMessage, WebhookMessages
from .webhook_payload import WebhookPayload



import logging

import pytest
from fastapi import Depends, FastAPI, status
from fastapi.testclient import TestClient
from telegram_wallet_pay.schemas import WebhookMessages
from telegram_wallet_pay.tools.fastapi import CheckSignature

logger = logging.getLogger(__name__)


@pytest.fixture
def app(token: str, path: str) -> FastAPI:
    """Prepare FastAPI application."""
    application = FastAPI()
    check_signature = CheckSignature(token)

    @application.post(path, dependencies=[Depends(check_signature)])
    async def webhook_handler(webhook_messages: WebhookMessages) -> dict[str, bool]:
        logger.info("Get webhook_messages: %s", webhook_messages)
        return {"success": True}

    return application


@pytest.fixture
def client(app: FastAPI) -> TestClient:
    """Prepare test client."""
    return TestClient(app)


def test_success(
    client: TestClient,
    signature: str,
    body: str,
    timestamp: str,
    path: str,
) -> None:
    """Test success case."""
    headers = {
        "WalletPay-Timestamp": timestamp,
        "WalletPay-Signature": signature,
    }
    response = client.post(path, headers=headers, content=body)
    assert response.status_code == status.HTTP_200_OK
    assert response.json() == {"success": True}


def test_timestamp_missing(
    client: TestClient,
    signature: str,
    path: str,
) -> None:
    headers = {
        "WalletPay-Signature": signature,
    }
    response = client.post(path, headers=headers)
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    assert response.json() == {"detail": "Timestamp header is missing"}


def test_timestamp_empty(
    client: TestClient,
    signature: str,
    path: str,
) -> None:
    headers = {
        "WalletPay-Timestamp": "",
        "WalletPay-Signature": signature,
    }
    response = client.post(path, headers=headers)
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    assert response.json() == {"detail": "Timestamp header is empty"}


def test_signature_missing(
    client: TestClient,
    path: str,
    timestamp: str,
) -> None:
    headers = {
        "WalletPay-Timestamp": timestamp,
    }
    response = client.post(path, headers=headers)
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    assert response.json() == {"detail": "Signature header is missing"}


def test_signature_empty(
    client: TestClient,
    path: str,
    timestamp: str,
) -> None:
    headers = {
        "WalletPay-Timestamp": timestamp,
        "WalletPay-Signature": "",
    }
    response = client.post(path, headers=headers)
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    assert response.json() == {"detail": "Signature header is empty"}


def test_signature_invalid(
    client: TestClient,
    path: str,
    timestamp: str,
) -> None:
    headers = {
        "WalletPay-Timestamp": timestamp,
        "WalletPay-Signature": "invalid-signature",
    }
    response = client.post(path, headers=headers)
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    assert response.json() == {"detail": "Signature is not valid"}



import os

import uvicorn
from fastapi import Depends, FastAPI
from telegram_wallet_pay.schemas import WebhookMessages
from telegram_wallet_pay.tools.fastapi import CheckSignature

# store TELEGRAM_WALLET_PAY_TOKEN to your .env
# wallet token can be issued via https://pay.wallet.tg/
TELEGRAM_WALLET_PAY_TOKEN = os.getenv("TELEGRAM_WALLET_PAY_TOKEN")

app = FastAPI()
check_signature = CheckSignature(TELEGRAM_WALLET_PAY_TOKEN)


@app.post("/wallet", dependencies=[Depends(check_signature)])
async def webhook_handler(webhook_messages: WebhookMessages) -> dict[str, bool]:
    """Handle webhook from Telegram Wallet API."""
    for webhook_message in webhook_messages:
        # process every webhook message as you wish
        # e.g. store them to your database. We just print it here
        print(f"Received webhook message: {webhook_message}")

    return {"success": True}


if __name__ == "__main__":
    uvicorn.run(app)



import logging
from collections.abc import Awaitable
from typing import Callable

import pytest
from aiohttp import web
from aiohttp.test_utils import TestClient
from telegram_wallet_pay.schemas import WebhookMessages
from telegram_wallet_pay.tools.aiohttp import check_signature

ClientFixture = Callable[[web.Application], Awaitable[TestClient]]
OK = 200
BAD_REQUEST = 400

logger = logging.getLogger(__name__)


@pytest.fixture
async def app(path: str, handler: Callable) -> web.Application:
    app = web.Application()
    app.router.add_route("POST", path, handler)
    yield app
    await app.cleanup()


@pytest.fixture
async def client(app: web.Application, aiohttp_client: ClientFixture) -> TestClient:
    client = await aiohttp_client(app)
    yield client
    await client.close()


@pytest.fixture
async def handler(token: str) -> Callable:
    @check_signature(token)
    async def handler(request: web.Request) -> web.StreamResponse:
        json_data = await request.text()
        webhook_messages = WebhookMessages.model_validate_json(json_data)
        logger.info("Got webhook messages: %s", webhook_messages)
        return web.json_response({"success": True})

    return handler


async def test_success(
    signature: str,
    body: str,
    timestamp: str,
    path: str,
    client: TestClient,
) -> None:
    """Test success case."""

    headers = {
        "WalletPay-Timestamp": timestamp,
        "WalletPay-Signature": signature,
    }
    response = await client.post(path=path, data=body, headers=headers)
    assert response.status == OK


async def test_timestamp_missing(
    signature: str,
    body: str,
    path: str,
    client: TestClient,
) -> None:
    """Test timestamp missing case."""
    headers = {
        "WalletPay-Signature": signature,
    }
    response = await client.post(path=path, data=body, headers=headers)
    assert response.status == BAD_REQUEST
    assert await response.json() == {"detail": "Timestamp header is missing"}


async def test_timestamp_empty(
    signature: str,
    body: str,
    path: str,
    client: TestClient,
) -> None:
    """Test timestamp is empty case."""

    headers = {
        "WalletPay-Timestamp": "",
        "WalletPay-Signature": signature,
    }
    response = await client.post(path=path, data=body, headers=headers)
    assert response.status == BAD_REQUEST
    assert await response.json() == {"detail": "Timestamp header is empty"}


async def test_signature_missing(
    body: str,
    timestamp: str,
    path: str,
    client: TestClient,
) -> None:
    """Test signature is missing case."""

    headers = {
        "WalletPay-Timestamp": timestamp,
    }
    response = await client.post(path=path, data=body, headers=headers)
    assert response.status == BAD_REQUEST
    assert await response.json() == {"detail": "Signature header is missing"}


async def test_signature_empty(
    body: str,
    timestamp: str,
    path: str,
    client: TestClient,
) -> None:
    """Test signature is empty case."""

    headers = {
        "WalletPay-Timestamp": timestamp,
        "WalletPay-Signature": "",
    }
    response = await client.post(path=path, data=body, headers=headers)
    assert response.status == BAD_REQUEST
    assert await response.json() == {"detail": "Signature header is empty"}


async def test_signature_invalid(
    body: str,
    timestamp: str,
    path: str,
    client: TestClient,
) -> None:
    """Test signature is invalid case."""

    headers = {
        "WalletPay-Timestamp": timestamp,
        "WalletPay-Signature": "invalid-signature",
    }
    response = await client.post(path=path, data=body, headers=headers)
    assert response.status == BAD_REQUEST
    assert await response.json() == {"detail": "Signature is not valid"}



import os

from aiohttp import web
from telegram_wallet_pay.schemas import WebhookMessages
from telegram_wallet_pay.tools.aiohttp import check_signature

# store TELEGRAM_WALLET_PAY_TOKEN to your .env
# wallet token can be issued via https://pay.wallet.tg/
TELEGRAM_WALLET_PAY_TOKEN = os.getenv("TELEGRAM_WALLET_PAY_TOKEN")


# add `check_signature` decorator to your handler
@check_signature(TELEGRAM_WALLET_PAY_TOKEN)
async def webhook_handler(request: web.Request) -> web.StreamResponse:
    """Handle webhook from Telegram Wallet API."""
    json_data = await request.text()
    webhook_messages = WebhookMessages.model_validate_json(json_data)
    for webhook_message in webhook_messages:
        # process every webhook message as you wish
        # e.g. store them to your database. We just print it here
        print(f"Received webhook message: {webhook_message}")

    return web.json_response({"success": True})


if __name__ == "__main__":
    app = web.Application()
    app.router.add_post("/wallet", webhook_handler)
    web.run_app(app)



from enum import Enum


class RequestStatus(str, Enum):
    SUCCESS = "SUCCESS"
    ALREADY = "ALREADY"
    CONFLICT = "CONFLICT"
    ACCESS_DENIED = "ACCESS_DENIED"
    INVALID_REQUEST = "INVALID_REQUEST"
    INTERNAL_ERROR = "INTERNAL_ERROR"



import asyncio
import os
from uuid import uuid4

from telegram_wallet_pay import TelegramWalletPay

# store TELEGRAM_WALLET_PAY_TOKEN to your .env
# wallet token can be issued via https://pay.wallet.tg/
TOKEN = os.getenv("TELEGRAM_WALLET_PAY_TOKEN")


async def main() -> None:
    """Create order."""
    # create wallet client instance
    wallet = TelegramWalletPay(TOKEN)

    # create your first order
    response = await wallet.create_order(
        amount=40,
        currency_code="EUR",
        description="TestPayment",
        external_id=str(uuid4()),
        timeout_seconds=5 * 60,
        customer_telegram_user_id=66812456,
    )

    # let's print creation response
    print("Response:", response)
    print("Order:", response.data)

    # don't forget close API-client instance on your app shutdown
    await wallet.close()


if __name__ == "__main__":
    asyncio.run(main())



from secrets import compare_digest
from typing import Optional

from fastapi import Depends, HTTPException, Request

from .signature import compute_signature


def get_timestamp(request: Request) -> str:
    """Get timestamp from header."""
    try:
        timestamp = request.headers["WalletPay-Timestamp"]
    except KeyError:
        raise HTTPException(
            status_code=400,
            detail="Timestamp header is missing",
        ) from None

    if not timestamp:
        raise HTTPException(
            status_code=400,
            detail="Timestamp header is empty",
        )

    return timestamp


def get_signature(request: Request) -> str:
    """Get signature from header."""
    try:
        signature = request.headers["WalletPay-Signature"]
    except KeyError:
        raise HTTPException(
            status_code=400,
            detail="Signature header is missing",
        ) from None

    if not signature:
        raise HTTPException(
            status_code=400,
            detail="Signature header is empty",
        )

    return signature


async def get_raw_body(request: Request) -> bytes:
    """Get request body."""
    return await request.body()


class CheckSignature:
    """Compare passed signature with computed signature."""

    def __init__(
        self,
        store_api_key: str,
        custom_exception: Optional[HTTPException] = None,
    ) -> None:
        self.__store_api_key = store_api_key
        self._exception = custom_exception or HTTPException(
            status_code=400,
            detail="Signature is not valid",
        )

    def __call__(
        self,
        request: Request,
        timestamp: str = Depends(get_timestamp),
        signature: str = Depends(get_signature),
        raw_body: bytes = Depends(get_raw_body),
    ) -> None:
        """Compare signatures."""
        valid_signature = compute_signature(
            store_api_key=self.__store_api_key,
            http_method=request.method,
            uri_path=request.url.path,
            timestamp=timestamp,
            body=raw_body,
        )

        if not compare_digest(signature, valid_signature):
            raise self._exception



from typing import Literal

from telegram_wallet_pay.enums import Currency

from ._default import DefaultModel


class MoneyAmount(DefaultModel):
    currency_code: Literal[
        Currency.TON,
        Currency.NOT,
        Currency.BTC,
        Currency.USDT,
        Currency.EUR,
        Currency.USD,
        Currency.RUB,
    ]
    amount: str



from typing import Union

import pytest
from telegram_wallet_pay.tools import compute_signature


# noinspection SpellCheckingInspection
@pytest.mark.parametrize(
    "body",
    [
        (
            '[{"eventDateTime":"2023-07-28T10:20:17.681338Z",'
            '"eventId":10030477545046017,"type":"ORDER_PAID","payload":{'
            '"id":10030467668508673,"number":"XYTNJP2O","customData":"in exercitation '
            'culpa","externalId":"JDF23NN","orderAmount":{"amount":"0.100000340",'
            '"currencyCode":"TON"},"selectedPaymentOption":{"amount":{'
            '"amount":"0.132653","currencyCode":"USDT"},"amountFee":{'
            '"amount":"0.001327","currencyCode":"USDT"},"amountNet":{'
            '"amount":"0.131326","currencyCode":"USDT"},'
            '"exchangeRate":"1.3265247467314987"},'
            '"orderCompletedDateTime":"2023-07-28T10:20:17.628946Z"}}]'
        ),
        (
            b'[{"eventDateTime":"2023-07-28T10:20:17.681338Z",'
            b'"eventId":10030477545046017,"type":"ORDER_PAID","payload":{'
            b'"id":10030467668508673,"number":"XYTNJP2O","customData":"in exercitation '
            b'culpa","externalId":"JDF23NN","orderAmount":{"amount":"0.100000340",'
            b'"currencyCode":"TON"},"selectedPaymentOption":{"amount":{'
            b'"amount":"0.132653","currencyCode":"USDT"},"amountFee":{'
            b'"amount":"0.001327","currencyCode":"USDT"},"amountNet":{'
            b'"amount":"0.131326","currencyCode":"USDT"},'
            b'"exchangeRate":"1.3265247467314987"},'
            b'"orderCompletedDateTime":"2023-07-28T10:20:17.628946Z"}}]'
        ),
    ],
)
def test_signature(body: Union[str, bytes]) -> None:
    """Test signature provided by docs."""
    signature = compute_signature(
        store_api_key="your_secret_api_key_sYIpNypce5sls6Ik",
        http_method="POST",
        uri_path="/webhook/",
        timestamp="168824905680291",
        body=body,
    )

    assert signature == "MGfJzeEprADZbihhRcGcCY5pYTI/IEJ91ejyA+XOWAs="


def test_empty_key(body: Union[str, bytes]) -> None:
    """Test signature provided by docs."""
    with pytest.raises(
        expected_exception=ValueError,
        match="Argument 'store_api_key' should not be empty.*",
    ):
        compute_signature(
            store_api_key="",
            http_method="POST",
            uri_path="/webhook/",
            timestamp="168824905680291",
            body=body,
        )



from ._default import DefaultModel
from .money_amount import MoneyAmount


class PaymentOption(DefaultModel):
    amount: MoneyAmount
    amount_fee: MoneyAmount
    amount_net: MoneyAmount
    exchange_rate: str



__version__ = "1.0.0"
__api_version__ = "1.2.0"
__all__ = [
    "TelegramWalletPay",
    "constants",
    "enums",
    "errors",
    "schemas",
    "tools",
]

from . import constants, enums, errors, schemas, tools
from .client import TelegramWalletPay



from typing import Literal, Optional

from telegram_wallet_pay.enums import RequestStatus

from ._default import DefaultModel
from .order_amount import OrderAmount


class OrderAmountResponse(DefaultModel):
    status: Literal[
        RequestStatus.SUCCESS,
        RequestStatus.INVALID_REQUEST,
        RequestStatus.INTERNAL_ERROR,
    ]
    message: Optional[str] = None
    data: Optional[OrderAmount] = None



import json
from collections.abc import AsyncIterator
from datetime import datetime
from typing import Any

import pytest
from aresponses import ResponsesMockServer
from telegram_wallet_pay import TelegramWalletPay
from telegram_wallet_pay.errors import InvalidAPIKeyError
from telegram_wallet_pay.schemas import (
    CreateOrderResponse,
    GetOrderPreviewResponse,
    GetOrderReconciliationListResponse,
    MoneyAmount,
    OrderAmount,
    OrderAmountResponse,
    OrderPreview,
    OrderReconciliationItem,
    OrderReconciliationList,
)

ORDER_PREVIEW = OrderPreview(
    id="",
    status="ACTIVE",
    number="",
    amount=MoneyAmount(currency_code="EUR", amount="42.0"),
    created_datetime=datetime.now(),
    expiration_datetime=datetime.now(),
    pay_link="",
    direct_pay_link="",
)

CREATE_ORDER_RESPONSE = CreateOrderResponse(
    status="SUCCESS",
    message="",
    data=ORDER_PREVIEW,
)

GET_ORDER_PREVIEW_RESPONSE = GetOrderPreviewResponse(
    status="SUCCESS",
    message="",
    data=ORDER_PREVIEW,
)

ORDER_RECONCILIATION_ITEM = OrderReconciliationItem(
    id=42,
    status="EXPIRED",
    amount=MoneyAmount(currency_code="EUR", amount="42.0"),
    external_id="",
    created_datetime=datetime.now(),
    expiration_datetime=datetime.now(),
)

GET_ORDERS_LIST_RESPONSE = GetOrderReconciliationListResponse(
    status="SUCCESS",
    message=None,
    data=OrderReconciliationList(items=[]),
)

ORDER_AMOUNT_RESPONSE = OrderAmountResponse(
    status="SUCCESS",
    message="",
    data=OrderAmount(total_amount=42),
)


@pytest.fixture
async def wallet() -> AsyncIterator[TelegramWalletPay]:
    """Prepare TelegramWalletPay fixture."""
    wallet = TelegramWalletPay("TOKEN")
    yield wallet
    await wallet.close()


class TestCreateOrder:
    METHOD = "POST"
    URI = "/wpay/store-api/v1/order"

    async def test_success(
        self,
        wallet: TelegramWalletPay,
        aresponses: ResponsesMockServer,
    ) -> None:
        """Test successful Order creation."""
        aresponses.add(
            path_pattern=self.URI,
            method_pattern=self.METHOD,
            response=aresponses.Response(
                text=CREATE_ORDER_RESPONSE.model_dump_json(by_alias=True),
                content_type="application/json",
                status=200,
            ),
        )
        response = await wallet.create_order(
            amount=ORDER_PREVIEW.amount.amount,
            currency_code=ORDER_PREVIEW.amount.currency_code,
            description="description",
            external_id="",
            timeout_seconds=30,
            customer_telegram_user_id=42,
        )
        assert response == CREATE_ORDER_RESPONSE
        aresponses.assert_plan_strictly_followed()


class TestGetOrderPreview:
    METHOD = "GET"
    URI = "/wpay/store-api/v1/order/preview"

    async def test_success(
        self,
        wallet: TelegramWalletPay,
        aresponses: ResponsesMockServer,
    ) -> None:
        """Test successful getting Order preview."""
        aresponses.add(
            path_pattern=self.URI,
            method_pattern=self.METHOD,
            response=aresponses.Response(
                text=GET_ORDER_PREVIEW_RESPONSE.model_dump_json(by_alias=True),
                content_type="application/json",
                status=200,
            ),
        )
        response = await wallet.get_order_preview(ORDER_PREVIEW.id)
        assert response == GET_ORDER_PREVIEW_RESPONSE
        aresponses.assert_plan_strictly_followed()

    async def test_invalid_token(
        self,
        wallet: TelegramWalletPay,
        aresponses: ResponsesMockServer,
    ) -> None:
        """Test successful getting Order preview."""
        aresponses.add(
            path_pattern=self.URI,
            method_pattern=self.METHOD,
            response=aresponses.Response(
                text=json.dumps({"success": False, "error": "Invalid token"}),
                content_type="application/json",
                status=401,
            ),
        )
        with pytest.raises(InvalidAPIKeyError):
            await wallet.get_order_preview(ORDER_PREVIEW.id)

        aresponses.assert_plan_strictly_followed()

    async def test_deprecated(
        self,
        wallet: TelegramWalletPay,
        aresponses: ResponsesMockServer,
    ) -> None:
        """Test getting Order preview with deprecated method."""
        aresponses.add(
            path_pattern=self.URI,
            method_pattern=self.METHOD,
            response=aresponses.Response(
                text=GET_ORDER_PREVIEW_RESPONSE.model_dump_json(by_alias=True),
                content_type="application/json",
                status=200,
            ),
        )
        with pytest.warns(DeprecationWarning):
            response = await wallet.get_preview(ORDER_PREVIEW.id)

        assert response == GET_ORDER_PREVIEW_RESPONSE
        aresponses.assert_plan_strictly_followed()


class TestGetOrderList:
    METHOD = "GET"
    URI = "/wpay/store-api/v1/reconciliation/order-list"

    async def test_success(
        self,
        wallet: TelegramWalletPay,
        aresponses: ResponsesMockServer,
    ) -> None:
        """Test successful getting Order preview."""
        aresponses.add(
            path_pattern=self.URI,
            method_pattern=self.METHOD,
            response=aresponses.Response(
                text=GET_ORDERS_LIST_RESPONSE.model_dump_json(by_alias=True),
                content_type="application/json",
                status=200,
            ),
        )
        response = await wallet.get_order_list(offset=0, count=10)
        assert response == GET_ORDERS_LIST_RESPONSE
        aresponses.assert_plan_strictly_followed()


class TestGetOrderAmount:
    METHOD = "GET"
    URI = "/wpay/store-api/v1/reconciliation/order-amount"

    async def test_success(
        self,
        wallet: TelegramWalletPay,
        aresponses: ResponsesMockServer,
    ) -> None:
        """Test successful getting Order preview."""
        aresponses.add(
            path_pattern=self.URI,
            method_pattern=self.METHOD,
            response=aresponses.Response(
                text=ORDER_AMOUNT_RESPONSE.model_dump_json(by_alias=True),
                content_type="application/json",
                status=200,
            ),
        )
        response = await wallet.get_order_amount()
        assert response == ORDER_AMOUNT_RESPONSE
        aresponses.assert_plan_strictly_followed()


class TestSession:
    async def test_close_without_session(self, wallet: TelegramWalletPay) -> None:
        """Test session close without any request."""


@pytest.mark.parametrize("token", [None, bool, 42])
def test_client_init_without_token(token: Any) -> None:
    """Check passed token is string and not empty."""
    msg = f"String token should be provided. You passed: {token!r}"
    with pytest.raises(RuntimeError, match=msg):
        # noinspection PyTypeChecker
        TelegramWalletPay(token)



"""Webhook IP security module.

Inspired by aiogram's webhook security module:
https://github.com/aiogram/aiogram

Docs:
https://docs.wallet.tg/pay/#operation/completedOrder
"""

from collections.abc import Sequence
from ipaddress import IPv4Address, IPv4Network
from typing import Any, Union

DEFAULT_WALLET_WEBHOOK_IPS = (
    IPv4Address("172.255.248.29"),
    IPv4Address("172.255.248.12"),
)


class IPFilter:
    """Tool for adding and checking allowed IP addresses."""

    def __init__(
        self,
        ips: Sequence[
            Union[str, IPv4Network, IPv4Address]
        ] = DEFAULT_WALLET_WEBHOOK_IPS,
    ) -> None:
        self._allowed_ips: set[IPv4Address] = set()

        if ips:
            self.allow_ip(*ips)

    def allow_ip(self, *ips: Union[str, IPv4Network, IPv4Address]) -> None:
        """Add IP or network to allowed list."""
        for ip in ips:
            ip_list = self._convert_ips(ip)
            self._allowed_ips.update(ip_list)

    def check(self, ip: Union[str, IPv4Address]) -> bool:
        """Check IP is allowed."""
        if not isinstance(ip, IPv4Address):
            ip = IPv4Address(ip)
        return ip in self._allowed_ips

    def __contains__(self, item: Union[str, IPv4Address]) -> bool:
        """Check IP is in allowed list."""
        return self.check(item)

    @staticmethod
    def _convert_ips(ip: Any) -> list[IPv4Address]:  # noqa: ANN401
        """Convert passed data to IP list."""
        if isinstance(ip, str):
            ip = IPv4Network(ip) if "/" in ip else IPv4Address(ip)

        if isinstance(ip, IPv4Address):
            return [ip]

        if isinstance(ip, IPv4Network):
            return list(ip.hosts())

        msg = f"Invalid type of IP address: {type(ip)} ('{ip}')"
        raise TypeError(msg)



from datetime import datetime
from typing import Literal, Optional

from pydantic import Field

from telegram_wallet_pay.enums import Currency, OrderStatus

from ._default import DefaultModel
from .money_amount import MoneyAmount
from .payment_option import PaymentOption


class OrderReconciliationItem(DefaultModel):
    id: int
    status: Literal[
        OrderStatus.ACTIVE,
        OrderStatus.EXPIRED,
        OrderStatus.PAID,
        OrderStatus.CANCELLED,
    ]
    amount: MoneyAmount
    auto_conversion_currency: Optional[
        Literal[
            Currency.TON,
            Currency.NOT,
            Currency.BTC,
            Currency.USDT,
        ]
    ] = None
    external_id: str
    customer_telegram_user_id: Optional[int] = None
    created_datetime: datetime = Field(alias="createdDateTime")
    expiration_datetime: datetime = Field(alias="expirationDateTime")
    payment_datetime: Optional[datetime] = Field(None, alias="paymentDateTime")
    selected_payment_option: Optional[PaymentOption] = None



import asyncio
import logging
import sys
from os import getenv
from uuid import uuid4

# install popular Telegram Bot API framework:
#   pip install aiogram
from aiogram import Bot, Dispatcher, Router, html
from aiogram.enums import ParseMode
from aiogram.types import InlineKeyboardButton, Message
from aiogram.utils.keyboard import InlineKeyboardBuilder

# install Telegram Wallet Pay API client library:
#   pip install telegram-wallet-pay
from telegram_wallet_pay import TelegramWalletPay
from telegram_wallet_pay.enums import ButtonName

# store TELEGRAM_BOT_API_TOKEN to your .env
# bot token can be issued via https://t.me/BotFather
TELEGRAM_BOT_API_TOKEN = getenv("TELEGRAM_BOT_API_TOKEN")

# store TELEGRAM_WALLET_PAY_TOKEN to your .env
# wallet token can be issued via https://pay.wallet.tg/
TELEGRAM_WALLET_PAY_TOKEN = getenv("TELEGRAM_WALLET_PAY_TOKEN")

# message handlers should be attached to the Router, so let's create it
router = Router()


@router.message()
async def message_handler(message: Message, wallet: TelegramWalletPay) -> None:
    """Handle any received message."""
    # prepare answer text
    header = html.bold("Welcome to Telegram Wallet API example")
    body = "Click on the button below to make payment"

    # create an order
    wallet_response = await wallet.create_order(
        amount=4.2,
        currency_code="USD",
        description="Example payment description",
        external_id=str(uuid4()),
        timeout_seconds=24 * 60 * 60,  # 1 day
        customer_telegram_user_id=message.from_user.id,
    )
    order = wallet_response.data

    # prepare keyboard
    keyboard = InlineKeyboardBuilder()
    wallet_button = InlineKeyboardButton(
        text=ButtonName.PAY_VIA_WALLET,
        url=order.pay_link,
    )
    keyboard.add(wallet_button)

    # send answer message with prepared content and keyboard markup
    await message.answer(
        text=f"{header}\n{body}",
        reply_markup=keyboard.as_markup(),
        parse_mode=ParseMode.HTML,
    )


async def main() -> None:
    """Set up and run bot application."""
    # create wallet client
    wallet = TelegramWalletPay(TELEGRAM_WALLET_PAY_TOKEN)

    # create dispatcher and pass wallet client to every bot handler
    dp = Dispatcher(wallet=wallet)

    # include router with handlers
    dp.include_router(router)

    # initialize Bot instance
    bot = Bot(TELEGRAM_BOT_API_TOKEN)

    # start polling Telegram Bot API
    try:
        await dp.start_polling(bot)
    finally:  # gracefully close sessions on stop
        await wallet.close()
        await bot.session.close()


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, stream=sys.stdout)
    asyncio.run(main())



ORDER_AMOUNT = {
    "currencyCode": "EUR",
    "amount": "40.00",
}

SELECTED_PAYMENT_OPTION = {
    "amount": {
        "currencyCode": "TON",
        "amount": "0.069848094",
    },
    "amountFee": {
        "currencyCode": "TON",
        "amount": "0.000698481",
    },
    "amountNet": {
        "currencyCode": "TON",
        "amount": "0.069149613",
    },
    "exchangeRate": "0.0017462023476135017",
}

WEBHOOK_PAYLOAD = {
    "id": 10829787081217,
    "number": "E953D09Q",
    "externalId": "0120bb34-5a74-4eb2-a3da-774b97cf3fbe",
    "orderAmount": ORDER_AMOUNT,
    "selectedPaymentOption": SELECTED_PAYMENT_OPTION,
    "orderCompletedDateTime": "2024-04-21T15:04:24.092Z",
}

WEBHOOK_MESSAGE_ORDER_PAID = {
    "eventDateTime": "2024-04-21T15:04:24.092Z",
    "eventId": 10829789207553,
    "type": "ORDER_PAID",
    "payload": WEBHOOK_PAYLOAD,
}

# noinspection SpellCheckingInspection
WEBHOOK_MESSAGE_ORDER_FAILED = {
    "eventId": 11044361216001,
    "eventDateTime": "2024-05-01T07:53:56.000991Z",
    "payload": {
        "id": 11000119772673,
        "number": "LVDPW6K8",
        "status": "EXPIRED",
        "customData": "some-custom-data",
        "externalId": "07d9539e-fd61-497e-a164-d2e354a55744",
        "orderAmount": {
            "amount": "0.44",
            "currencyCode": "USD",
        },
        "orderCompletedDateTime": "2024-05-01T07:53:56.000980Z",
    },
    "type": "ORDER_FAILED",
}

WEBHOOK_MESSAGES = [WEBHOOK_MESSAGE_ORDER_PAID, WEBHOOK_MESSAGE_ORDER_FAILED]



from enum import Enum

from telegram_wallet_pay.constants import WALLET_EMOJI


class ButtonName(str, Enum):
    """Enum with available button names.

    The payment button should be named exactly like one of these names.

    Source:
    https://docs.wallet.tg/pay/#section/Design-Guidelines
    """

    WALLET_PAY = f"{WALLET_EMOJI} Wallet Pay"
    PAY_VIA_WALLET = f"{WALLET_EMOJI} Pay via Wallet"



class TelegramWalletPayError(Exception):
    """Base TelegramWalletPay exception."""


class InvalidRequestError(TelegramWalletPayError):
    """Invalid request error."""


class InvalidAPIKeyError(TelegramWalletPayError):
    """Invalid API key error."""


class NotFountError(TelegramWalletPayError):
    """Not found error."""


class RequestLimitReachedError(TelegramWalletPayError):
    """Request limit reached error."""


class UnexpectedError(TelegramWalletPayError):
    """Unexpected error."""



import logging
import sys
from os import getenv
from uuid import uuid4

# install popular Telegram Bot API framework:
#   pip install aiogram
from aiogram import Bot, Dispatcher, Router, html
from aiogram.enums import ParseMode
from aiogram.types import InlineKeyboardButton, Message
from aiogram.utils.keyboard import InlineKeyboardBuilder
from aiogram.webhook.aiohttp_server import SimpleRequestHandler, setup_application
from aiohttp import web

# install Telegram Wallet Pay API client library:
#   pip install telegram-wallet-pay
from telegram_wallet_pay import TelegramWalletPay
from telegram_wallet_pay.enums import ButtonName, WebhookMessageType
from telegram_wallet_pay.schemas import WebhookMessage, WebhookMessages
from telegram_wallet_pay.tools.aiohttp import check_signature

# store TELEGRAM_BOT_API_TOKEN to your .env
# bot token can be issued via https://t.me/BotFather
TELEGRAM_BOT_API_TOKEN = getenv("TELEGRAM_BOT_API_TOKEN")

# store TELEGRAM_WALLET_PAY_TOKEN to your .env
# wallet token can be issued via https://pay.wallet.tg/
TELEGRAM_WALLET_PAY_TOKEN = getenv("TELEGRAM_WALLET_PAY_TOKEN")

# Set your server domain name (do not forget to set up SSL)
WEBHOOK_HOST = getenv("WEBHOOK_HOST", "https://example.com")

# Set up paths for webhooks
TELEGRAM_BOT_WEBHOOK_PATH = getenv("TELEGRAM_BOT_WEBHOOK_PATH", "/bot")
TELEGRAM_WALLET_WEBHOOK_PATH = getenv("TELEGRAM_WALLET_WEBHOOK_PATH", "/wallet")

# Create bot handlers router
router = Router()


# Prepare message handler
@router.message()
async def message_handler(message: Message, wallet: TelegramWalletPay) -> None:
    """Handle any received message."""
    # prepare answer text
    header = html.bold("Welcome to Telegram Wallet API example")
    body = "Click on the button below to make payment"

    # create an order
    user = message.from_user
    wallet_response = await wallet.create_order(
        amount=4.2,
        currency_code="EUR",
        description="Example payment description",
        external_id=str(uuid4()),
        timeout_seconds=24 * 60 * 60,  # 1 day
        customer_telegram_user_id=user.id,
        custom_data=str(user.id),
    )
    order = wallet_response.data

    # prepare keyboard
    keyboard = InlineKeyboardBuilder()
    wallet_button = InlineKeyboardButton(
        text=ButtonName.PAY_VIA_WALLET,
        url=order.pay_link,
    )
    keyboard.add(wallet_button)

    # send answer message with prepared content and keyboard markup
    await message.answer(
        text=f"{header}\n{body}",
        reply_markup=keyboard.as_markup(),
        parse_mode=ParseMode.HTML,
    )


# Add set webhook on bot startup function
async def on_bot_startup(bot: Bot) -> None:
    """Set up bot before start up."""
    await bot.set_webhook(f"{WEBHOOK_HOST}{TELEGRAM_BOT_WEBHOOK_PATH}")


# Prepare wallet webhook handler
@check_signature(TELEGRAM_WALLET_PAY_TOKEN)
async def wallet_webhook_handler(request: web.Request) -> web.StreamResponse:
    """Handle webhook from Telegram Wallet API."""
    bot: Bot = request.app["bot"]

    json_data = await request.text()
    webhook_messages = WebhookMessages.model_validate_json(json_data)

    for webhook_message in webhook_messages:
        await _process_payment(webhook_message, bot)

    return web.json_response({"success": True})


async def _process_payment(webhook_message: WebhookMessage, bot: Bot) -> None:
    """Process payment."""
    order = webhook_message.payload
    user_id = int(order.custom_data)

    if webhook_message.type == WebhookMessageType.ORDER_PAID:
        text = "Thanks, order has been paid"
    else:
        text = f"Order status was changed to {order.status}"

    await bot.send_message(chat_id=user_id, text=text)


def main() -> None:
    """Set up and run bot application."""
    dp = Dispatcher()
    dp.include_router(router)
    dp.startup.register(on_bot_startup)

    # Create bot instance
    bot = Bot()

    # Create wallet client
    wallet = TelegramWalletPay(TELEGRAM_WALLET_PAY_TOKEN)

    # Create aiohttp.web.Application instance
    app = web.Application()

    # Pass bot instance to the application to make it available in aiohttp handlers
    app["bot"] = bot

    # Register Telegram Bot API webhook handler
    webhook_requests_handler = SimpleRequestHandler(
        dispatcher=dp,
        bot=bot,
        wallet=wallet,  # pass wallet here to make it available in bot handlers
    )
    webhook_requests_handler.register(app, path=TELEGRAM_BOT_WEBHOOK_PATH)
    setup_application(app, dp, bot=bot)

    # Register Wallet Pay API webhook handler
    app.router.add_post(TELEGRAM_WALLET_WEBHOOK_PATH, wallet_webhook_handler)

    # And finally start webserver
    web.run_app(app, host="127.0.0.1", port=8080)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, stream=sys.stdout)
    main()



from datetime import datetime
from typing import Optional

from pydantic import Field

from telegram_wallet_pay.enums import OrderStatus

from ._default import DefaultModel
from .money_amount import MoneyAmount
from .payment_option import PaymentOption


class WebhookPayload(DefaultModel):
    id: int
    number: str
    external_id: str = Field(max_length=255)
    status: Optional[OrderStatus] = None
    custom_data: Optional[str] = Field(None, max_length=255)
    order_amount: MoneyAmount
    selected_payment_option: Optional[PaymentOption] = None
    order_completed_datetime: datetime = Field(alias="orderCompletedDateTime")



import hashlib
import hmac
from base64 import b64encode
from typing import Union

ENCODING = "utf-8"


def compute_signature(
    store_api_key: str,
    http_method: str,
    uri_path: str,
    timestamp: Union[str, float],
    body: Union[str, bytes],
) -> str:
    """Compute signature."""
    if not store_api_key:
        msg = f"Argument 'store_api_key' should not be empty. Passed {store_api_key=}"
        raise ValueError(msg)

    if isinstance(body, str):
        body = bytes(body, ENCODING)

    body_base64 = b64encode(body).decode(ENCODING)
    payload = f"{http_method}.{uri_path}.{timestamp}.{body_base64}"

    signature_bytes = hmac.new(
        key=bytes(store_api_key, ENCODING),
        msg=bytes(payload, ENCODING),
        digestmod=hashlib.sha256,
    ).digest()

    return b64encode(signature_bytes).decode(ENCODING)
